var winwidth;		//窗口宽度
var winheight;		//窗口高度

/**
 * 是否是正整数
 * @param str
 * @returns
 */
function isPInt(str){
	return /^[1-9]*[1-9][0-9]*$/.test(str);
}

/**
 * 是否是整数
 * @param str
 * @returns
 */
function isInt(str){
	return /^-?\d+$/.test(str);
}

/**
 * 是否为小数
 * @param str
 * @returns
 */
function isFloat(str){
	return /^-?\d+\.\d+$/.test(str);
}
/**
 * 是否为正小数
 * @param str
 * @returns
 */
function isPFloat(str){
	return /^\d+\.\d+$/.test(str);
}

/**
 * 浮点数相加
 * @param a
 * @param b
 * @param fixed			小数位数，-1或不传为返回原始值
 * @param retainzero	保留0，必须和fixed一起使用
 * @returns	retainzero为真返回字符串，否之返回浮点数
 */
function floatadd(a, b, fixed, retainzero) {
    var c, d, e, re;
    try {
        c = a.toString().split(".")[1].length;
    } catch (f) {
        c = 0;
    }
    try {
        d = b.toString().split(".")[1].length;
    } catch (f) {
        d = 0;
    }
    e = Math.pow(10, Math.max(c, d));
    re = (floatmul(a, e) + floatmul(b, e)) / e;
    if( isInt(fixed) && fixed>=0 ){
    	re = retainzero ? re.toFixed(fixed) : re.toFixed(fixed) - 0;
    }
    return re;
}
/**
 * 浮点数相减，参数同上
 */
function floatsub(a, b, fixed, retainzero) {
    var c, d, e, re;
    try {
        c = a.toString().split(".")[1].length;
    } catch (f) {
        c = 0;
    }
    try {
        d = b.toString().split(".")[1].length;
    } catch (f) {
        d = 0;
    }
    e = Math.pow(10, Math.max(c, d));
    re = (floatmul(a, e) - floatmul(b, e)) / e;
    if( isInt(fixed) && fixed>=0 ){
    	re = retainzero ? re.toFixed(fixed) : re.toFixed(fixed) - 0;
    }
    return re;
}
/**
 * 浮点数相乘，参数同上
 */
function floatmul(a, b, fixed, retainzero) {
    var c = 0,
        d = a.toString(),
        e = b.toString(),
        re;
    try {
        c += d.split(".")[1].length;
    } catch (f) {}
    try {
        c += e.split(".")[1].length;
    } catch (f) {}
    re = Number(d.replace(".", "")) * Number(e.replace(".", "")) / Math.pow(10, c);
    if( isInt(fixed) && fixed>=0 ){
    	re = retainzero ? re.toFixed(fixed) : re.toFixed(fixed) - 0;
    }
    return re;
}


/**
 * 获取url参数
 * @param name
 * @returns
 */
function getUrlParam(name) {
	var reg = new RegExp("(^|&)" + name + "=([^&]*)(&|$)"); //构造一个含有目标参数的正则表达式对象
	var r = window.location.search.substr(1).match(reg); //匹配目标参数
	if (r != null) 
		return unescape(r[2]);
	return false; //返回参数值
}

/**
 * 添加参数到url
 */
function addParameterToUrl(url, param_name, value, is_return){
	if( url=='' ){
		url = location.href;
	}
	url = removeParameter(url, param_name);
	url += url.indexOf('?')>=0 ? '&'+param_name+'='+value : '?'+param_name+'='+value;
	if( typeof(is_return)=='undefined' || is_return ){
		return url;
	}
	location.href = url;
}

/**
 * 删除url的参数
 */
function removeParameter(url, param_name){
	var re = new RegExp("(\\?|&)"+param_name+"=[^&]*(&)?","gim");
	return url.replace(re, function(p0, p1, p2) {
		return p1 === '?' || p2 ? p1 : '';
	});
}

/**
 * 判断微信浏览器
 */
function isWeixin(){
    var ua = navigator.userAgent.toLowerCase();
    if(ua.match(/MicroMessenger/i)=="micromessenger") {
        return true;
    } else {
        return false;
    }
}

/**
 * 提示框
 * @param	string		msg				消息
 * @param	int			showTime		显示时间
 * @param	function	onHideHandler	隐藏后回调
 */
window.Alert = function(msg, showTime, onHideHandler){
	var $dom = $('<div class="window-alert"><p>'+msg+'</p></div>');
	$('body').append($dom);
	if( typeof showTime == 'undefined' ){
		showTime = 1500;
	}else{
		showTime = Math.max( parseInt(showTime), 500 );
	}
	var i=setTimeout(function(){
		clearTimeout(i);
		setTimeout(function(){
			$dom.remove();
			if( typeof(onHideHandler) == 'function'){
				onHideHandler();
			}
		}, 500);
		$dom.css('opacity', 0);
	}, showTime);
}

/**
 * 确认框
 * @param	string		msg				消息
 * @param	function	onHideHandler	隐藏后回调
 * @param	array		btnTxtArr		按钮文本文字
 */
window.Confirm = function(msg, onHideHandler, btnTxtArr){
	var txt = typeof btnTxtArr == 'object' && btnTxtArr.length==2 ? btnTxtArr : ['取消', '确定'];
	var $dom = $('<div class="window-confirm"><div class="wrap"><p>'+msg+'</p><div><a class="cancel" href="javascript:;">'+txt[0]+'</a><a class="ok" href="javascript:;">'+txt[1]+'</a></div></div></div>');
	$('body').append($dom);
	var $a = $dom.find('a').on('click', function(){
		var $span = $(this);
		setTimeout(function(){
			$dom.remove();
			if( typeof onHideHandler == 'function' ){
				onHideHandler( $span.hasClass('ok') );
			}
		}, 500);
		$dom.css('opacity', 0);
	}).eq(1);
	$a.focus();
};


$(function(){
	//ios iframe下table滑动超宽BUG解决
	if( window.isiframe && window.device == 'ios' ){
		var $tablebox = $('.table-responsive-list-box');
		if( $tablebox.length > 0 ){
			var $table = $tablebox.children('table');
			$table.hide();
			$tablebox.each(function(){
				$(this).css('width', $(this).width());
			})
			$table.show();
			if( window.pack !== 'app' ){
				$(window).on('winresize', function(){
					$table.hide();
					$tablebox.each(function(){
						$(this).css('width', 'auto').css('width', $(this).width());
					});
					$table.show();
				});
			}
		}
	}
	
	//移动解决300mm延时
	if(window.pack=='app') FastClick.attach(document.body);
	
	//ios打包下iframe输入框无法自动失去焦点
	if( window.device=='ios' && window.pack=='app' ) {
		$(document).click(function(e){
			if( e.target.nodeName != 'INPUT' && 
				e.target.nodeName != 'SELECT' && 
				e.target.nodeName != 'TEXTAREA' && 
				(document.activeElement.nodeName == 'INPUT' || document.activeElement.nodeName == 'SELECT' || document.activeElement.nodeName == 'TEXTAREA') ){
				document.activeElement.blur(); 
				//调用键盘插件
				var win = window.isiframe ? window.parent : window;
				win.Keyboard.hide();
			}
		});
	}
	
	winwidth = $(window).width();
	winheight = $(window).height();
	
	/**
     * 切换Input组件
     */
    $('.input-switch').click(function(){
        var $this = $(this);
        if( $this.hasClass('open') ){
            $this.removeClass('open');
            $this.trigger('switch', {'type':'close'});
        }else{
            $this.addClass('open');
            $this.trigger('switch', {'type':'open'});
        }
    });

    /**
     * 点击发送短信验证码
     */
    $('.send-smscode').click(function(){
        var $btn = $(this);
        if( $btn.hasClass('ing') ){
            return;
        }	
        
        //验证短信
        var $form     = $btn.parents('form'),
            mobile_re = /^1\d{10}$/,
            mobile    = $form.find('input[name="mobile"]').val();
            
        //是否有验证码
        var $captchaInput = $form.find('input[name="captcha"]');
        if( $captchaInput.length>0 ){
            var captchacode = $captchaInput.val();
            if( captchacode.length < 3 ){
                return Alert('请填写正确的图形验证码', 1000);
            }
        }
        
        if( !mobile_re.test(mobile) ){
            return Alert('请填写正确的手机号码', 1000);
        }
        
        $btn.addClass('ing');
        //发送
        window.request('post', $btn.data('url'), {mobile:mobile, captcha:captchacode}, false, function(data){
        	if( !data ){
        		$btn.text('获取验证码').removeClass('ing')
        		return Alert('网络错误，请重试');
        	}
        	
        	if( !data.code || data.code!=1 ){
                 $btn.text('获取验证码').removeClass('ing');
                 Alert( data.message ? data.message : '发送失败', 1000 , function(){
                     var $img = $form.find(".captcha-img");
                     if( $img.length>0 ){
                    	 $form.find('.show-piccode').click();
                     }
                 });
                 return false;
             }
             Alert('发送成功', 1000);
             var i = 120;
             var n = setInterval(function(){
                 $btn.text(i+'秒后重发');
                 if(i==0){
                     clearInterval(n);
                     $btn.html('获取验证码').removeClass('ing');
                 }
                 i--;
             }, 1000);
    	});
    });
    
    //点击显示弹出层
    $('.btn-showpopup').click(function(){
    	var $this = $(this);
		$(window).trigger('popup-show-click', {popupname:$this.data('popup'), popupdata:$this.data('popupdata')});
	});	
	//隐藏弹出层
	$('.popup').click(function(e){
		if( e.target == e.currentTarget ){
			$(window).trigger('popup-mask-click', {popup:this});
		}
	});
	
	//筛选层
	$('.btn-minscreen-filter').click(function(){
		$($(this).data('filtername')).toggleClass('show');
	});
	$('.minscreen-filter').click(function(e){
		if( e.target == e.currentTarget ) $(this).removeClass('show');
	});
	
	//加减框
	$('.num-plusminus-group .input-group-btn').click(function(){
		var $this = $(this)
			index = $this.index(),
			$input= null;
		if( index == 0 ){
			//减
			$input = $(this).next().children('input');
		}else{
			//加
			$input = $(this).prev().children('input');
		}
		var val = $input.val();
		if( !isInt(val) && !isFloat(val) ) val='0';
		
		var arr = val.split('.');
		var step = arr.length == 1 ? 1 : 0.1; 
		var fixlen = arr.length > 1 && arr[1].length == 1 ? 1 : 2;
		
		if( index == 0 ){
			val = arr.length > 1 ? (parseFloat(val)-step).toFixed(fixlen) : parseInt(val)-step;
		}else{
			val = arr.length > 1 ? (parseFloat(val)+step).toFixed(fixlen) : parseInt(val)+step;
		}
		
		$input.val( val );
	});
	
	//退出
	$('.trigger-logout').click(function(){
		window.request('get', $(this).data('url'), null, false, function(data){        	
    		if( !data.code || data.code != 1 ) return Alert(data.message ? data.message : '操作失败');
    		setMemberData({});
    		if( window.pack && window.pack == 'windows' ){
    			winExit();
    		}else{
    			location.href = window.login_url;
    		}
    	});
	});
});

if( typeof(avalon) != 'undefined' ){
	avalon.config({
	    interpolate: ['{?', '?}'],
	    debug: true
	});
}



//====================用户数据=======================
function setMemberData(member){
	localStorage.setItem('member', JSON.stringify(member));
}
function getMemberData(){
	var member = JSON.parse(localStorage.getItem('member'));
	window.memberId = member ? member['id'] : 0;
	window.memberMobile = member ? member['mobile'] : 0;
	window.memberNickname = member ? member['nickname'] : 0;
	window.merchantId = member ? member['merchant_id'] : 0;
	window.merchantName = member ? member['merchant_name'] : '';
	window.merchantPhone = member ? member['merchant_phone'] : '';
	window.settings = member ? member['settings'] : null;
	window.accessToken = member ? member['access_token'] : '';
	if( !window.memberId ){
		if( window.login_url ){
			location.href = window.login_url;
		}
	}
}
getMemberData();


//====================win基础=======================
//win端接口
function winApi(json_params, call, isAsync) {
    $.ajax({
        type: 'GET',
        async: isAsync===false ? false : true,
        url: location.origin + "/local_api",
        data: JSON.stringify(json_params),
        dataType: 'json',
        contentType: 'application/json;charset=utf-8',
        success: call
    });
}

//win端设置Webpath
function winInitWebPath(){
	winApi({
        mod:"win32",
        func:"web_path"
    },function (x) {
        window.win_rest = x.status;
        window.win_web_path = x.data;
    }, false);
}

//win端打印
function winPrint(content){
	if( !window.win_web_path ){
		winInitWebPath();
	}
	winApi({
      mod: "win32",
      func: "print",
      src:"<html><head>" +
      "<meta http-equiv=\"content-type\" content=\"text/html;charset=utf-8\">" +
      "<title>itest</title></head><body>"+
      "<div style='text-align:center'><img src='" + winFilePath("images\\print-logo.png")+ "' width='100'/></div>"+content+"</body></html>"
  }, function(x){
      console.info(x);
  });
}

//win退出
function winExit(){
	winApi({
        mod: "win32",
        func: "exit"
    }, function(x){
        console.info(x)
        self.rest = x    
    })
}

//win端设置本地文件路径
function winFilePath (file) {
    return "file:///" + window.win_web_path + "\\" + file;
}


/**
 * =====================请求======================
 * @param	string	type	类别(post,get)
 * @param	string	url		地址
 * @param	array	params	参数
 * @param	bool	cache	是否缓存
 * @param	fun		callback回调
 */
window.request = function(type, url, params, cache, callback){
	cache = cache ? true : false;
	type = type == 'post' ? 'post' : 'get';
	
	//将本地token传入
	if( !params ) params = {};
	params['access_token'] = window.accessToken;
	
	//验证回传数据
	function checkresult(issuccess, obj){
		if( !issuccess ) return callback(false, obj);
		
		//设置token
		if( obj.token && obj.token != window.accessToken ){
			window.accessToken = obj.token;
        }
		
		//验证是否重新登录
		if( obj.code && obj.code == 1000 ){
            location.href = window.login_url;
        }
		
		callback(obj);
	}
	
	//APP
	if( window.pack && window.pack == 'app' ){
		var win = window.isiframe ? window.parent : window;
		win.document.addEventListener('deviceready', function () {
			win.cordova.plugin.http[type](url, params, {"X-Requested-With":"XMLHttpRequest"},function(result){
				checkresult(true, JSON.parse(result.data));//成功
			},function(error){
				checkresult(false, error);//失败
			});
		});
		return;
	}
	
	//win及浏览器
	$.ajax({
        url: url,
        type: type,
        data: params,
        dataType: 'json',
        cache: cache,
        error: function(error){
        	checkresult(false, error);
        },
        success: function(result){
        	checkresult(true, result);
        }
    });
}

//返回处理
function returnHandler(data, opstring){
	var code	= data && data.code ? data.code : 0,
		message	= data && data.message ? data.message : '',
		data	= data && data.data ? data.data : {};
		
	//错误
	if( code == 0 ){
		Alert( message ? message : (opstring?opstring:'获取数据')+'失败');
		return false;
	}
	
	//跳转成功
	if( code == 1101 ){
		location.href = data;
		return false;
	}
	
	//未登录
	if( code == 1000 ){
		Alert( message ? message : '登录过期，请重新登录', 1000, function(){
			if( window.login_url ){
				//location.href = window.login_url;
			}
		});
		return false;
	}
	
	//返回
	return {code:code, message:message, data:data};
}

//====================打印=======================
function toPrint(type, params){
	params['_pack'] = window.pack;
	window.request('get', window.serv_root+'/printer/'+type, params, false, function(data){        	
		if( !data.code || data.code != 1 ){
            return Alert(data.message ? data.message : '获取打印数据失败');
        }
		if( window.pack == 'windows' ){
			return winPrint(data.data);
		}
		if( window.pack == 'app' ){
			var win = window.isiframe ? window.parent : window;
			win.cordova.exec(function (result){
                var obj = JSON.parse(result);
                if(obj.type == 0){
                	Alert(obj.msg);
                }
            },function (msg){
                Alert(msg);
            },"THPlugin","print",[{'type':type, 'data':data.data}]);
		}
	});
}
//匹配打印机
function checkPrinter(){
	var win = window.isiframe ? window.parent : window;
	win.cordova.exec(function (result){
        var obj = JSON.parse(result);
        if(obj.type == 0){
        	Alert(obj.msg);
        }
    },function (msg){
        Alert(msg);
    },"THPlugin","choicePrinter",[]);
}
//原生回调Js
function cordovaToJs(name, type, msg, data){
	//获取到蓝牙打印机状态
	if( name == 'getPrintStatus' ){
		//TODO
	}
}